class RobustPCA
{
public:
  int N;
  int K;

  int max_iter = 100;

  Real lm1, lm2;

  Real ep;

  MatrixXr L;
  VectorXr r;
  VectorXr x, e;

  MatrixXr A, B;

  explicit
  RobustPCA(int N=257, int K=100) :
    N(N), K(K)
  {
    ep = 1e-4;

    lm1 = 1 / sqrt(N) * 8e-2;
    lm2 = 1 / sqrt(N) * 8e-2;
  }

  explicit
  RobustPCA(const RobustPCA &rpca) :
    N(rpca.N), K(rpca.K),
    lm1(rpca.lm1), lm2(rpca.lm2), ep(rpca.ep),
    L(rpca.L), r(rpca.r), x(rpca.x), e(rpca.e),
    A(rpca.A), B(rpca.A)
  {
    initialize();
  }

  void initialize(void)
  {
    L = MatrixXr::Random(N, K).cwiseAbs();

    r = VectorXr::Random(K).cwiseAbs();
    e = VectorXr::Random(N).cwiseAbs();

    A = MatrixXr::Zero(K, K);
    B = MatrixXr::Zero(N, K);
  }

  template<typename Any>
  void update(const Eigen::MatrixBase<Any> &z) {
    // Step 2
    MatrixXr L_ = L.transpose() * L;
    L_ += VectorXr::Constant(K, lm1).asDiagonal();
    L_ = (L_.inverse() * L.transpose()).eval();

    int it = 0;
    VectorXr r_, e_;
    do {
      r_ = r, e_ = e; // back up

      r = L_ * (z - e);

      e = z - L * r;
      for (int n=0;n<N;n++) {
	if (e(n) > lm2) e(n) -= lm2;
	else if (e(n) < -lm2) e(n) += lm2;
	else e(n) = 0;
      }

      it++;
      if (it > max_iter) {
	std::cout << "not converged" << std::endl;
	break;
      }
      // std::cout << it << " " << (max((r - r_).norm(), (e - e_).norm())) << std::endl;
    } while (!(max<Real>((r - r_).norm(), (e - e_).norm()) < ep * z.norm()));
    
    // Step 3
    A += r * r.transpose();
    B += (z - e) * r.transpose();

    // Step 4
    MatrixXr A_ = A;
    A_ += VectorXr::Constant(K, lm1).asDiagonal();
    for(int k=0;k<K;k++){
      L.col(k) += (1 / A_(k, k) * (B.col(k) - L * A_.col(k))).eval();
    }

    x = L * r;
  }
};
